@@ -2,11 +2,11 @@ use crate::input::actions::Action;
 use crate::input::config::ConversionError;
 use clap::ArgEnum;
 use serde::{Deserialize, Serialize};
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::fmt;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::str::FromStr;
-use strum_macros::{EnumDiscriminants, EnumIter, EnumString, ToString};
+use strum_macros::{Display, EnumDiscriminants, EnumIter, EnumString, ToString};
 
 pub type ClientId = u16; // TODO: merge with crate type?
 
@@ -493,6 +493,64 @@ pub enum Event {
     FileSystemUpdate(Vec<PathBuf>),
     /// A file was deleted somewhere in the Zellij CWD folder
     FileSystemDelete(Vec<PathBuf>),
+    /// A Result of plugin permission request
+    PermissionRequestResult(PermissionStatus),
+    SessionUpdate(Vec<SessionInfo>),
+}
+
+#[derive(
+    Debug,
+    PartialEq,
+    Eq,
+    Hash,
+    Copy,
+    Clone,
+    EnumDiscriminants,
+    ToString,
+    Serialize,
+    Deserialize,
+    PartialOrd,
+    Ord,
+)]
+#[strum_discriminants(derive(EnumString, Hash, Serialize, Deserialize, Display, PartialOrd, Ord))]
+#[strum_discriminants(name(PermissionType))]
+#[non_exhaustive]
+pub enum Permission {
+    ReadApplicationState,
+    ChangeApplicationState,
+    OpenFiles,
+    RunCommands,
+    OpenTerminalsOrPlugins,
+    WriteToStdin,
+}
+
+impl PermissionType {
+    pub fn display_name(&self) -> String {
+        match self {
+            PermissionType::ReadApplicationState => {
+                "Access Zellij state (Panes, Tabs and UI)".to_owned()
+            },
+            PermissionType::ChangeApplicationState => {
+                "Change Zellij state (Panes, Tabs and UI)".to_owned()
+            },
+            PermissionType::OpenFiles => "Open files (eg. for editing)".to_owned(),
+            PermissionType::RunCommands => "Run commands".to_owned(),
+            PermissionType::OpenTerminalsOrPlugins => "Start new terminals and plugins".to_owned(),
+            PermissionType::WriteToStdin => "Write to standard input (STDIN)".to_owned(),
+        }
+    }
+}
+
+#[derive(Debug, Clone)]
+pub struct PluginPermission {
+    pub name: String,
+    pub permissions: Vec<PermissionType>,
+}
+
+impl PluginPermission {
+    pub fn new(name: String, permissions: Vec<PermissionType>) -> Self {
+        PluginPermission { name, permissions }
+    }
 }
 
 /// Describes the different input modes, which change the way that keystrokes will be interpreted.
@@ -677,6 +735,42 @@ impl ModeInfo {
     }
 }
 
+#[derive(Debug, Default, Clone, PartialEq, Eq, Deserialize, Serialize)]
+pub struct SessionInfo {
+    pub name: String,
+    pub tabs: Vec<TabInfo>,
+    pub panes: PaneManifest,
+    pub connected_clients: usize,
+    pub is_current_session: bool,
+}
+
+use std::hash::{Hash, Hasher};
+
+#[allow(clippy::derive_hash_xor_eq)]
+impl Hash for SessionInfo {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.name.hash(state);
+    }
+}
+
+impl SessionInfo {
+    pub fn new(name: String) -> Self {
+        SessionInfo {
+            name,
+            ..Default::default()
+        }
+    }
+    pub fn update_tab_info(&mut self, new_tab_info: Vec<TabInfo>) {
+        self.tabs = new_tab_info;
+    }
+    pub fn update_pane_info(&mut self, new_pane_info: PaneManifest) {
+        self.panes = new_pane_info;
+    }
+    pub fn update_connected_clients(&mut self, new_connected_clients: usize) {
+        self.connected_clients = new_connected_clients;
+    }
+}
+
 /// Contains all the information for a currently opened tab.
 #[derive(Debug, Default, Clone, PartialEq, Eq, Hash, Deserialize, Serialize)]
 pub struct TabInfo {
@@ -810,3 +904,161 @@ pub enum CopyDestination {
     Primary,
     System,
 }
+
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
+pub enum PermissionStatus {
+    Granted,
+    Denied,
+}
+
+#[derive(Debug, Default, Clone)]
+pub struct FileToOpen {
+    pub path: PathBuf,
+    pub line_number: Option<usize>,
+    pub cwd: Option<PathBuf>,
+}
+
+impl FileToOpen {
+    pub fn new<P: AsRef<Path>>(path: P) -> Self {
+        FileToOpen {
+            path: path.as_ref().to_path_buf(),
+            ..Default::default()
+        }
+    }
+    pub fn with_line_number(mut self, line_number: usize) -> Self {
+        self.line_number = Some(line_number);
+        self
+    }
+    pub fn with_cwd(mut self, cwd: PathBuf) -> Self {
+        self.cwd = Some(cwd);
+        self
+    }
+}
+
+#[derive(Debug, Default, Clone)]
+pub struct CommandToRun {
+    pub path: PathBuf,
+    pub args: Vec<String>,
+    pub cwd: Option<PathBuf>,
+}
+
+impl CommandToRun {
+    pub fn new<P: AsRef<Path>>(path: P) -> Self {
+        CommandToRun {
+            path: path.as_ref().to_path_buf(),
+            ..Default::default()
+        }
+    }
+    pub fn new_with_args<P: AsRef<Path>, A: AsRef<str>>(path: P, args: Vec<A>) -> Self {
+        CommandToRun {
+            path: path.as_ref().to_path_buf(),
+            args: args.into_iter().map(|a| a.as_ref().to_owned()).collect(),
+            ..Default::default()
+        }
+    }
+}
+
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+pub struct ConnectToSession {
+    pub name: Option<String>,
+    pub tab_position: Option<usize>,
+    pub pane_id: Option<(u32, bool)>, // (id, is_plugin)
+}
+
+#[derive(Debug, Default, Clone)]
+pub struct PluginMessage {
+    pub name: String,
+    pub payload: String,
+    pub worker_name: Option<String>,
+}
+
+impl PluginMessage {
+    pub fn new_to_worker(worker_name: &str, message: &str, payload: &str) -> Self {
+        PluginMessage {
+            name: message.to_owned(),
+            payload: payload.to_owned(),
+            worker_name: Some(worker_name.to_owned()),
+        }
+    }
+    pub fn new_to_plugin(message: &str, payload: &str) -> Self {
+        PluginMessage {
+            name: message.to_owned(),
+            payload: payload.to_owned(),
+            worker_name: None,
+        }
+    }
+}
+
+#[derive(Debug, Clone, EnumDiscriminants, ToString)]
+#[strum_discriminants(derive(EnumString, Hash, Serialize, Deserialize))]
+#[strum_discriminants(name(CommandType))]
+pub enum PluginCommand {
+    Subscribe(HashSet<EventType>),
+    Unsubscribe(HashSet<EventType>),
+    SetSelectable(bool),
+    GetPluginIds,
+    GetZellijVersion,
+    OpenFile(FileToOpen),
+    OpenFileFloating(FileToOpen),
+    OpenTerminal(FileToOpen),         // only used for the path as cwd
+    OpenTerminalFloating(FileToOpen), // only used for the path as cwd
+    OpenCommandPane(CommandToRun),
+    OpenCommandPaneFloating(CommandToRun),
+    SwitchTabTo(u32), // tab index
+    SetTimeout(f64),  // seconds
+    ExecCmd(Vec<String>),
+    PostMessageTo(PluginMessage),
+    PostMessageToPlugin(PluginMessage),
+    HideSelf,
+    ShowSelf(bool), // bool - should float if hidden
+    SwitchToMode(InputMode),
+    NewTabsWithLayout(String), // raw kdl layout
+    NewTab,
+    GoToNextTab,
+    GoToPreviousTab,
+    Resize(Resize),
+    ResizeWithDirection(ResizeStrategy),
+    FocusNextPane,
+    FocusPreviousPane,
+    MoveFocus(Direction),
+    MoveFocusOrTab(Direction),
+    Detach,
+    EditScrollback,
+    Write(Vec<u8>), // bytes
+    WriteChars(String),
+    ToggleTab,
+    MovePane,
+    MovePaneWithDirection(Direction),
+    ClearScreen,
+    ScrollUp,
+    ScrollDown,
+    ScrollToTop,
+    ScrollToBottom,
+    PageScrollUp,
+    PageScrollDown,
+    ToggleFocusFullscreen,
+    TogglePaneFrames,
+    TogglePaneEmbedOrEject,
+    UndoRenamePane,
+    CloseFocus,
+    ToggleActiveTabSync,
+    CloseFocusedTab,
+    UndoRenameTab,
+    QuitZellij,
+    PreviousSwapLayout,
+    NextSwapLayout,
+    GoToTabName(String),
+    FocusOrCreateTab(String),
+    GoToTab(u32),                    // tab index
+    StartOrReloadPlugin(String),     // plugin url (eg. file:/path/to/plugin.wasm)
+    CloseTerminalPane(u32),          // terminal pane id
+    ClosePluginPane(u32),            // plugin pane id
+    FocusTerminalPane(u32, bool),    // terminal pane id, should_float_if_hidden
+    FocusPluginPane(u32, bool),      // plugin pane id, should_float_if_hidden
+    RenameTerminalPane(u32, String), // terminal pane id, new name
+    RenamePluginPane(u32, String),   // plugin pane id, new name
+    RenameTab(u32, String),          // tab index, new name
+    ReportPanic(String),             // stringified panic
+    RequestPluginPermissions(Vec<PermissionType>),
+    SwitchSession(ConnectToSession),
+}
