@@ -338,6 +338,11 @@ pub enum ScreenContext {
     FocusPaneWithId,
     RenamePane,
     RenameTab,
+    RequestPluginPermissions,
+    BreakPane,
+    BreakPaneRight,
+    BreakPaneLeft,
+    UpdateSessionInfos,
 }
 
 /// Stack call representations corresponding to the different types of [`PtyInstruction`]s.
@@ -374,6 +379,7 @@ pub enum PluginContext {
     PostMessageToPluginWorker,
     PostMessageToPlugin,
     PluginSubscribedToEvents,
+    PermissionRequestResult,
 }
 
 /// Stack call representations corresponding to the different types of [`ClientInstruction`]s.
@@ -391,6 +397,7 @@ pub enum ClientContext {
     OwnClientId,
     StartedParsingStdinQuery,
     DoneParsingStdinQuery,
+    SwitchSession,
 }
 
 /// Stack call representations corresponding to the different types of [`ServerInstruction`]s.
@@ -408,11 +415,15 @@ pub enum ServerContext {
     ConnStatus,
     ActiveClients,
     Log,
+    SwitchSession,
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
 pub enum PtyWriteContext {
     Write,
+    ResizePty,
+    StartCachingResizes,
+    ApplyCachedResizes,
     Exit,
 }
 
@@ -421,6 +432,8 @@ pub enum BackgroundJobContext {
     DisplayPaneError,
     AnimatePluginLoading,
     StopPluginLoadingAnimation,
+    ReadAllSessionInfosOnMachine,
+    ReportSessionInfo,
     Exit,
 }
 
